/**
 * $Id: connqueue.h 1 2006-04-29 18:51:20Z alexeen $
 */

#ifndef __CONNQUEUE_H__
#define __CONNQUEUE_H__

#include <netinet/in.h>
#include <pthread.h>

/**
 * This is just an element of the connection queue.
 * Represents a socket that needs to be handled by a worker
 * thread.
 */
typedef struct _connqueue_elm_t
{
	int sock; /** Socket in current connection queue element. **/
	struct sockaddr_in *addr; /** Socket address. **/
	struct _connqueue_elm_t *next; /** The next in queue. **/
} connqueue_elm_t;

/**
 * Note: users of the queue should lock the mutex
 *   before manipulating the queue! Race conditions must be
 *   explicitly prevented, functions of the queue alone are
 *   not thread safe.
 */
typedef struct _connqueue_t
{
	/** Mutex locks to make this thread safe. **/
	pthread_mutexattr_t *cq_mta;
	pthread_mutex_t *cq_mutex;

	/** Connection queue elements. **/
	struct _connqueue_elm_t *head;
	struct _connqueue_elm_t *tail;
} connqueue_t;

void cq_init(connqueue_t *cq);
void cq_enqueue(int sock, struct sockaddr_in *addr, connqueue_t *cq);
int cq_dequeue(struct sockaddr_in *addr, connqueue_t *cq);
int cq_is_empty(connqueue_t *cq);
void cq_destroy(connqueue_t *cq);

#endif /** __CONNQUEUE_H__ **/
