/**
 * $Id: opts.h 1 2006-04-29 18:51:20Z alexeen $
 */
#ifndef __OPTS_H__
#define __OPTS_H__

#include "common.h"

/** Flag values in HEX. **/
#define F_NORMAL	0x00
#define F_HELP		0x01
#define F_VERSION	0x02
#define F_DEBUG		0x04
#define F_IDENT   0x08

/** Flags setting function. **/
#define SET_NORMAL(arg) (arg |= F_NORMAL)
#define SET_HELP(arg) (arg |= F_HELP)
#define SET_VERSION(arg) (arg |= F_VERSION)
#define SET_DEBUG(arg) (arg |= F_DEBUG)
#define SET_IDENT(arg) (arg |= F_IDENT)

/** Flags checking functions. **/
#define IS_NORMAL(arg) (F_NORMAL == (arg & F_NORMAL) ? 1 : 0)
#define IS_HELP(arg) (F_HELP == (arg & F_HELP) ? 1 : 0)
#define IS_VERSION(arg) (F_VERSION == (arg & F_VERSION) ? 1 : 0)
#define IS_DEBUG(arg) (F_DEBUG == (arg & F_DEBUG) ? 1 : 0)
#define IS_IDENT(arg) (F_IDENT == (arg & F_IDENT) ? 1 : 0)

/**
 * Structure holding program runtime options.
 */
typedef struct _opts_t
{
	/** Stores the remote server host/IP. **/
	char rhost[BUFLEN];

	/** Stores the remote server port. **/
	int rport;

	/** Stores the listening port. **/
	int lport;

	/**
	 * Bit	Flag
	 *  0   Normal Mode
 	 *  1		Help (Display usage & exit)
	 *  2		Version (Display version & exit)
	 *  3		Debug Mode (Run in foreground with output to stdout/stderr)
	 *  4		Enable passing IDENT to remote server
	 */
	short flags:16;
} opts_t;

void opts_init(opts_t *opts);
int opts_get(opts_t *opts, int argc, char **argv);

#endif /** __OPTS_H__ **/
