/**
 * $Id: resolvehost.c 1 2006-04-29 18:51:20Z alexeen $
 */
#include <stdlib.h>
#include <netdb.h>
#include <errno.h>
#include <assert.h>

#include "resolvehost.h"
#include "common.h"

/**
 * resolve_host: Resolves a host address specified as a string.
 *   This is mainly providing an easy way to do thread-safe host
 *   name resolution. Not that we require on the fly resolution
 *   anyway with how the program is structured.
 * @param host - Host specified as a string.
 * @param he - The hostent structure storage.
 * @param buf - The buffer used for resolution (>= 1024 bytes).
 * @param buflen - Length of the buffer (>= 1024).
 * @return Returns 0 for success, -1 otherwise.
 */
int resolve_host(char *host, struct hostent *rhostent, char *buf, size_t buflen)
{
	struct hostent *result = rhostent;
	int res;
	int herr;

	assert(host);
	assert(rhostent);
	assert(buf);
	assert(buflen >= 1024); /** At least this much. **/

	res = gethostbyname_r(host, rhostent, buf, buflen, &result, &herr);

	/** Check for errors. **/
	if (res || NULL == result)
	{
		return -1;
	}

	return 0;
}
