/**
 * $Id: sendall.c 1 2006-04-29 18:51:20Z alexeen $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <assert.h>

#include "common.h"

/**
 * sendall: Sends the data in buffer fully out the socket.
 * Note: this code is from Beej's Guide to Network
 *       Programming Using Internet Sockets
 * @param sock - The socket file descriptor to send to.
 * @param buf - The data buffer to send.
 * @param buflen - The length of the data buffer.
 * @param retval - Stores return value of this function.
 * @return Returns -1 on failure, 0 on success.
 */
int sendall(int sock, char *buf, int buflen, int *retval)
{
	int nb_sent = 0; /** Num bytes sent. **/
	int nb_remaining = buflen; /** Num bytes remaining. **/
	int nbytes = -1; /** Number of bytes sent per pass. **/

	assert(retval);
	assert(0 < buflen);
	assert(0 <= sock);
	assert(buf);

	/** Keep sending until we have sent all of it. **/
	while(nb_sent < buflen)
	{
		/** Send it out. **/
		nbytes = send(sock, buf+nb_sent, nb_remaining, 0);
		if (nbytes == -1) { break; } /** Errored, stop immediately. **/
		nb_sent += nbytes; /** Accumulate sent counter. **/
		nb_remaining -= nbytes; /** Decrement remaining counter. **/
	}

	*retval = nb_sent; /** Return value for number of bytes sent. **/

	/** Returns -1 on failure, 0 on success. **/
	return nbytes == -1 ? -1 : 0;
} 
