/**
 * $Id: workers.h 1 2006-04-29 18:51:20Z alexeen $
 */
#ifndef __WORKERS_H__
#define __WORKERS_H__

/**
 * Workforce status bit in octal.
 */
#define WF_RUN	000 /** Running **/
#define WF_TERM 001 /** Terminate **/

/** Check the workforce status. **/
#define WF_IS_TERM(arg) (WF_TERM == (arg & WF_TERM) ? 1 : 0)

/**
 * Data structure representing a workforce.
 * Mainly holds information about the number of worker threads,
 * termination status, pointers to thread pool, conditional variables,
 * and mutex locks.
 */
typedef struct _workforce_t
{
	long num_workers;     /** Number of workers in pool. **/
	char terminate;       /** Denotes termination of workforce. **/

	pthread_t *pool;      /** List of threads in thread pool. **/

	pthread_mutexattr_t *wf_mta;  /** Mutex attribute. **/
	pthread_condattr_t *wf_cda;   /** CV attribute. **/

	/** Worker threads should wait on conditional variable for dispatch. **/
	pthread_mutex_t *wf_mutex; /** Mutex lock for conditional variable. **/
	pthread_cond_t *wf_cv;     /** Conditional variable for workers to wait on. **/

} workforce_t;

int wf_init(int n, workforce_t *wf, void *(*func)(void *));
int wf_dispatch(workforce_t *wf);
void wf_destroy(workforce_t *wf);

#endif /** __WORKERS_H__ **/
